package com.github.gkutiel.flip.db;

import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

public class ResultSetParser {

	private final Class<?> returnType;
	private final ResultSet rs;

	public ResultSetParser(final Class<?> type, final ResultSet rs) {
		returnType = type;
		this.rs = rs;
	}

	public Object parse() {
		try {
			if (returnType.equals(ResultSet.class)) return rs;
			if (returnType.isArray()) return parseArray();
			return parseObject();
		} catch (final Exception e) {
			throw new RuntimeException(e);
		}
	}

	private Object parseArray() throws SQLException {
		final Class<?> componentType = returnType.getComponentType();
		final List<Object> result = new ArrayList<>();
		while (rs.next())
			result.add(new ResultSetParser(componentType, rs).parse());
		return result.toArray((Object[]) Array.newInstance(componentType, result.size()));
	}

	private Object parseObject() throws SQLException, InstantiationException, IllegalAccessException {
		if (rs.isBeforeFirst()) rs.next();
		if (rs.isAfterLast()) return null;
		if (returnType.equals(String.class)) return rs.getString(1);
		if (returnType.equals(Integer.class)) return rs.getInt(1);
		if (returnType.equals(Long.class)) return rs.getLong(1);
		// TODO (gilad) handle other types as well
		final Object obj = returnType.newInstance();
		final Field[] fields = returnType.getDeclaredFields();
		AccessibleObject.setAccessible(fields, true);
		for (final Field field : fields)
			try {
				field.set(obj, rs.getObject(field.getName()));
			} catch (final SQLException | IllegalArgumentException | IllegalAccessException e) {}

		return obj;
	}
}
